@@ -21,16 +21,6 @@
 #ifndef FST_LIB_RMFINALEPSILON_H__
 #define FST_LIB_RMFINALEPSILON_H__
 
-/*#ifdef USE_TR1_UNORDERED_SET
-#include <tr1/unordered_set>
-using std::tr1::unordered_set;
-using std::tr1::unordered_multiset;
-#else
-#include <unordered_set>
-using std::unordered_set;
-using std::unordered_multiset;
-#endif*/
-
 #include <vector>
 using std::vector;
 
