@@ -66,7 +66,14 @@ pmatcherror(const char *msg)
 void pmatchwarning(const char *msg)
 {
     if (hfst::pmatch::verbose) {
-        std::cerr << "pmatch: "<< msg << std::endl;
+        std::string warnmsg = "pmatch: ";
+        warnmsg.append(msg);
+        warnmsg.append(" on line ");
+        std::ostringstream ss;
+        ss << pmatchlineno;
+        warnmsg.append(ss.str());
+        warnmsg.append("\n");
+        std::cerr << warnmsg;
     }
 }
 
@@ -84,12 +91,14 @@ std::set<std::string> inserted_names;
 std::set<std::string> unsatisfied_insertions;
 std::set<std::string> used_definitions;
 std::set<std::string> function_names;
+std::set<std::string> capture_names;
 std::vector<WordVector> word_vectors;
 char* startptr;
 hfst::ImplementationType format;
 size_t len;
 bool verbose;
 bool flatten;
+bool include_cosine_distances;
 std::string includedir;
 clock_t timer;
 int minimization_guard_count;
@@ -304,83 +313,122 @@ HfstTransducer * add_pmatch_delimiters(HfstTransducer * regex)
 PmatchTransducerContainer * make_end_tag(std::string tag)
 { return epsilon_to_symbol_container("@PMATCH_ENDTAG_" + tag + "@"); }
 
-struct DotProductWithWordVectorComparison {
-    WordVector compare_with_this;
-    DotProductWithWordVectorComparison(WordVector word): compare_with_this(word) {}
-    // if the vectors are normalized, dot product is == cosine similarity
-    bool operator()(WordVector left, WordVector right) {
-        WordVecFloat left_accumulator = 0;
-        WordVecFloat right_accumulator = 0;
-        for (size_t i = 0; i < compare_with_this.vector.size(); ++i) {
-            left_accumulator += compare_with_this.vector[i]*left.vector[i];
-            right_accumulator += compare_with_this.vector[i]*right.vector[i];
-        }
-        return left_accumulator > right_accumulator;
-    }
-};
+PmatchTransducerContainer * make_capture_tag(std::string tag)
+{ return epsilon_to_symbol_container("@PMATCH_CAPTURE_" + tag + "@"); }
 
-struct CosineSimilarityWithWordVectorComparison {
-    WordVector compare_with_this;
-    CosineSimilarityWithWordVectorComparison(WordVector word): compare_with_this(word) {}
-    bool operator()(WordVector left, WordVector right) {
-        WordVecFloat left_accumulator = 0.0;
-        WordVecFloat right_accumulator = 0.0;
-        for (size_t i = 0; i < compare_with_this.vector.size(); ++i) {
-            left_accumulator += compare_with_this.vector[i]*left.vector[i];
-            right_accumulator += compare_with_this.vector[i]*right.vector[i];
-        }
-        left_accumulator /= left.norm;
-        right_accumulator /= right.norm;
-        return left_accumulator > right_accumulator;
-    }
-};
+PmatchTransducerContainer * make_captured_tag(std::string tag)
+{ return epsilon_to_symbol_container("@PMATCH_CAPTURED_" + tag + "@"); }
 
-std::vector<WordVecFloat> get_projected_vector(std::vector<WordVecFloat> vec,
-                                               std::vector<WordVecFloat> plane_vec,
-                                               WordVecFloat translation_term)
+PmatchObject * make_with_tag_entry(std::string key, std::string value)
 {
-    return pointwise_plus(vec,
-                          pointwise_multiplication(
-                              (((translation_term - dot_product(vec, plane_vec)) / square_sum(plane_vec))
-                               * vector_similarity_projection_factor),
-                              plane_vec));
+    return new PmatchString("@P.PMATCH_GLOBAL_" + key + "." + value + "@");
 }
 
-struct CosineSimilarityProjectedToPlaneComparison {
-    std::vector<WordVecFloat> plane_vec;
-    std::vector<WordVecFloat> comparison_point;
-    WordVecFloat translation_term;
-    WordVecFloat plane_vec_square_sum;
-    CosineSimilarityProjectedToPlaneComparison(
-        std::vector<WordVecFloat> plane_vec_, std::vector<WordVecFloat> comparison_point_, WordVecFloat translation_term_):
-        plane_vec(plane_vec_), comparison_point(comparison_point_), translation_term(translation_term_)
-        {
-            plane_vec_square_sum = square_sum(plane_vec_);
+PmatchObject * make_with_tag_exit(std::string key)
+{
+    return new PmatchString("@C.PMATCH_GLOBAL_" + key + "@");
+}
+
+// Get the n best candidates in the original space using an insertion sort
+std::vector<std::pair<WordVector, WordVecFloat> > get_top_n(size_t n,
+                                                            std::vector<WordVector> & vecs,
+                                                            WordVector & comparison_point)
+{
+    std::vector<std::pair<WordVector, WordVecFloat> > retval;
+    for (std::vector<WordVector>::const_iterator it = vecs.begin();
+         it != vecs.end(); ++it) {
+        WordVecFloat cosdist = cosine_distance(*it, comparison_point);
+        for (size_t i = 0; i <= retval.size(); ++i) {
+            if (i == retval.size()) {
+                // We made it to the top
+                retval.push_back(std::pair<WordVector, WordVecFloat>(
+                                     WordVector(*it), cosdist));
+                break;
+            } else {
+                // Walking the list
+                if (cosdist >= retval[i].second) {
+                    if (i == 0 && retval.size() == n) {
+                        break;
+                    }
+                    retval.insert(retval.begin() + i, std::pair<WordVector, WordVecFloat>(
+                                      WordVector(*it), cosdist));
+                    break;
+                } else {
+                    continue;
+                }
+            }
         }
-    bool operator()(WordVector left, WordVector right) {
+        if (retval.size() > n) {
+            retval.erase(retval.begin());
+        }
+    }
+    return retval;    
+}
+
+// Get the n best candidates in the transformed space using an insertion sort
+std::vector<std::pair<WordVector, WordVecFloat> > get_top_n_transformed(
+    size_t n,
+    const std::vector<WordVector> & vecs,
+    std::vector<WordVecFloat> plane_vec,
+    std::vector<WordVecFloat> comparison_point,
+    WordVecFloat translation_term,
+    bool negative)
+{
+    std::vector<std::pair<WordVector, WordVecFloat> > retval;
+    WordVecFloat plane_vec_square_sum = square_sum(plane_vec);
+    WordVecFloat comparison_point_norm = norm(comparison_point);
+    for (std::vector<WordVector>::const_iterator it = vecs.begin();
+         it != vecs.end(); ++it) {
+        WordVector transformed_vec(*it);
+
         /*
          * First, given a plane "plane_vec = translation term" and a point,
          * find the multiple of plane_vec which produces a vector going
          * from point to the nearest point in the plane.
          */
-        WordVecFloat left_scaler = (translation_term - dot_product(left.vector, plane_vec)) / plane_vec_square_sum;
-        WordVecFloat right_scaler = (translation_term - dot_product(right.vector, plane_vec)) / plane_vec_square_sum;
-        left_scaler *= vector_similarity_projection_factor;
-        right_scaler *= vector_similarity_projection_factor;
-        std::vector<WordVecFloat> new_left = pointwise_plus(left.vector, pointwise_multiplication(left_scaler, plane_vec));
-        std::vector<WordVecFloat> new_right = pointwise_plus(right.vector, pointwise_multiplication(right_scaler, plane_vec));
-        // Then calculate cosine similarity
-        WordVecFloat left_accumulator = 0.0;
-        WordVecFloat right_accumulator = 0.0;
-        for (size_t i = 0; i < comparison_point.size(); ++i) {
-            left_accumulator += comparison_point[i]*new_left[i];
-            right_accumulator += comparison_point[i]*new_right[i];
+
+        WordVecFloat transformed_vec_scaler =
+            (translation_term - dot_product(transformed_vec.vector, plane_vec))
+            / plane_vec_square_sum;
+        transformed_vec_scaler *= vector_similarity_projection_factor;
+        if(negative) {
+            transformed_vec.vector =
+                pointwise_minus(transformed_vec.vector,
+                                pointwise_multiplication(transformed_vec_scaler, plane_vec));
+        } else {
+            transformed_vec.vector =
+                pointwise_plus(transformed_vec.vector,
+                               pointwise_multiplication(transformed_vec_scaler, plane_vec));
+        }
+        transformed_vec.norm = norm(transformed_vec.vector);
+        WordVecFloat cosdist = 1 - dot_product(transformed_vec.vector, comparison_point)
+            / (transformed_vec.norm * comparison_point_norm);
+        for (size_t i = 0; i <= retval.size(); ++i) {
+            if (i == retval.size()) {
+                // We made it to the top
+                retval.push_back(std::pair<WordVector, WordVecFloat>(transformed_vec,
+                                                                     cosdist));
+                break;
+            } else {
+                // Walking the list
+                if (cosdist >= retval[i].second) {
+                    if (i == 0 && retval.size() == n) {
+                        break;
+                    }
+                    retval.insert(retval.begin() + i,
+                                  std::pair<WordVector, WordVecFloat>(transformed_vec, cosdist));
+                    break;
+                } else {
+                    continue;
+                }
+            }
+        }
+        if (retval.size() > n) {
+            retval.erase(retval.begin());
         }
-        left_accumulator /= norm(new_left);
-        right_accumulator /= norm(new_right);
-        return left_accumulator > right_accumulator;
     }
-};
+    return retval;    
+}
 
 template<typename T> std::vector<T> pointwise_minus(std::vector<T> l,
                                                     std::vector<T> r)
@@ -442,23 +490,52 @@ WordVecFloat cosine_distance(WordVector left, WordVector right)
     // a slightly negative distance, so make sure to return at least 0.0
     WordVecFloat retval = 1.0 - dot_product(left.vector, right.vector) /
         (left.norm * right.norm);
-    if (retval < 0.0) {
-        return 0.0;       
-    }
-    return retval;
+    return std::max(static_cast<WordVecFloat>(0.0), retval);
 }
 
 WordVecFloat cosine_distance(std::vector<WordVecFloat> left, std::vector<WordVecFloat> right)
 {
     WordVecFloat retval = 1.0 - dot_product(left, right) / (norm(left) * norm(right));
-    if (retval < 0.0) {
-        return 0.0;       
-    }
-    return retval;
+    return std::max(static_cast<WordVecFloat>(0.0), retval);
 }
 
-PmatchObject * compile_like_arc(std::string word1, std::string word2,
+// Single-word Like()
+PmatchObject * compile_like_arc(std::string word,
                                 unsigned int nwords)
+{
+    WordVector this_word;
+    for (std::vector<WordVector>::iterator it = word_vectors.begin();
+         it != word_vectors.end(); ++it) {
+        if (word == it->word) {
+            this_word = *it;
+            break;
+        }
+    }
+    if (this_word.word == "") {
+        // got no matches
+        PmatchString * word_o = new PmatchString(word);
+        word_o->multichar = true;
+        pmatchwarning("no matches for argument to Like() operation");
+        return word_o;
+    }
+
+    std::vector<std::pair<WordVector, WordVecFloat> > top_n = get_top_n(nwords, word_vectors, this_word);
+
+    HfstTokenizer tok;
+    HfstTransducer * retval = new HfstTransducer(format);
+    for (size_t i = 0; i < top_n.size(); ++i) {
+        HfstTransducer tmp(top_n[i].first.word, tok, format);
+        if (include_cosine_distances) {
+            tmp.set_final_weights(top_n[i].second);
+        }
+        retval->disjunct(tmp);
+    }
+    return new PmatchTransducerContainer(retval);
+}
+
+// the general case
+PmatchObject * compile_like_arc(std::string word1, std::string word2,
+                                unsigned int nwords, bool is_negative)
 {
     WordVector this_word1;
     WordVector this_word2;
@@ -477,25 +554,27 @@ PmatchObject * compile_like_arc(std::string word1, std::string word2,
         PmatchString * word1_o = new PmatchString(word1);
         PmatchString * word2_o = new PmatchString(word2);
         word1_o->multichar = true; word2_o->multichar = true;
+        pmatchwarning("no matches for arguments to Like() operation");
         return new PmatchBinaryOperation(Disjunct, word1_o, word2_o);
     }
 
     if (this_word1.word == "" || this_word2.word == "") {
         // just one match
+        pmatchwarning("only one match for arguments to Like() operation, using nearest neighbours");
         WordVector this_word = (this_word1.word == "" ? this_word2 : this_word1);
-        CosineSimilarityWithWordVectorComparison comparison_object(this_word);
-        std::sort(word_vectors.begin(), word_vectors.end(), comparison_object);
+        std::vector<std::pair<WordVector, WordVecFloat> > top_n = get_top_n(nwords, word_vectors, this_word);
         HfstTokenizer tok;
         HfstTransducer * retval = new HfstTransducer(format);
-        for (size_t i = 0; i < word_vectors.size() && i <= nwords; ++i) {
-            HfstTransducer tmp(word_vectors[i].word, tok, format);
-            tmp.set_final_weights(cosine_distance(word_vectors[i], this_word));
+        for (size_t i = 0; i < top_n.size(); ++i) {
+            HfstTransducer tmp(top_n[i].first.word, tok, format);
+            if (include_cosine_distances) {
+                tmp.set_final_weights(top_n[i].second);
+            }
             retval->disjunct(tmp);
         }
         return new PmatchTransducerContainer(retval);
     }
 
-    // the general case
     if(variables["vector-similarity-projection-factor"] != "1.0") {
         vector_similarity_projection_factor =
             strtod(variables["vector-similarity-projection-factor"].c_str(), NULL);
@@ -513,33 +592,47 @@ PmatchObject * compile_like_arc(std::string word1, std::string word2,
      * translation term. |B - A| = 0 would be the set of vectors orthogonal to
      * |B - A|. We set d so that the distance from the hyperplane to A is
      * half of the norm of |B - A|.
+     *
      */
         
     std::vector<WordVecFloat> B_minus_A = pointwise_minus(
         this_word1.vector, this_word2.vector);
-    std::vector<WordVecFloat> halfway_point = pointwise_plus(
-        this_word2.vector, pointwise_multiplication(
-            static_cast<WordVecFloat>(0.5), B_minus_A));
     WordVecFloat hyperplane_translation_term = dot_product(B_minus_A, this_word1.vector)
         - square_sum(B_minus_A) * 0.5;
-    CosineSimilarityProjectedToPlaneComparison comparison_object(
-        B_minus_A, halfway_point, hyperplane_translation_term);
-    std::sort(word_vectors.begin(), word_vectors.end(), comparison_object);
 
+    std::vector<WordVecFloat> comparison_point;
+    if (is_negative == true) {
+        WordVecFloat comparison_scaler =
+            (hyperplane_translation_term - dot_product(this_word1.vector, B_minus_A)) / square_sum(B_minus_A);
+        comparison_scaler *= vector_similarity_projection_factor;
+        comparison_point = pointwise_minus(this_word1.vector, pointwise_multiplication(comparison_scaler, B_minus_A));
+    } else {
+        comparison_point = pointwise_plus(this_word2.vector, pointwise_multiplication(
+                                              static_cast<WordVecFloat>(0.5), B_minus_A));
+    }
+    
+    std::vector<std::pair<WordVector, WordVecFloat> > top_n = get_top_n_transformed(nwords,
+                                                                                    word_vectors,
+                                                                                    B_minus_A,
+                                                                                    comparison_point,
+                                                                                    hyperplane_translation_term,
+                                                                                    is_negative);
     HfstTokenizer tok;
     HfstTransducer * retval = new HfstTransducer(format);
-    for (size_t i = 0; i < word_vectors.size() && i <= nwords; ++i) {
-        HfstTransducer tmp(word_vectors[i].word, tok, format);
-        std::vector<WordVecFloat> projected_i = get_projected_vector(
-            word_vectors[i].vector, B_minus_A, hyperplane_translation_term);
-        tmp.set_final_weights(cosine_distance(projected_i, halfway_point));
-        retval->disjunct(tmp);
-        for (size_t j = i + 1; j < word_vectors.size() && j <= nwords; ++j) {
-            HfstTransducer tmp2(word_vectors[i].word + "_cos_" + word_vectors[j].word, tok, format);
-            tmp2.set_final_weights(cosine_distance(projected_i,
-                                                   get_projected_vector(word_vectors[j].vector, B_minus_A, hyperplane_translation_term)));
-            retval->disjunct(tmp2);
+    for (size_t i = 0; i < top_n.size() && i <= nwords; ++i) {
+        HfstTransducer tmp(top_n[i].first.word, tok, format);
+        if (include_cosine_distances) {
+            tmp.set_final_weights(top_n[i].second);
         }
+        retval->disjunct(tmp);
+        // if (include_cosine_distances) {
+        //     for (size_t j = i + 1; j < word_vectors.size() && j <= nwords; ++j) {
+        //         HfstTransducer tmp2(word_vectors[i].word + "_cos_" + word_vectors[j].word, tok, format);
+        //         tmp2.set_final_weights(cosine_distance(projected_i,
+        //                                                get_projected_vector(word_vectors[j].vector, B_minus_A, hyperplane_translation_term)));
+        //         retval->disjunct(tmp2);
+        //     }
+        // }
     }
     return new PmatchTransducerContainer(retval);
 }
@@ -547,18 +640,27 @@ PmatchObject * compile_like_arc(std::string word1, std::string word2,
 PmatchTransducerContainer * make_counter(std::string name)
 { return epsilon_to_symbol_container("@PMATCH_COUNTER_" + name + "@"); }
 
+hfst::StringSet get_non_special_alphabet(HfstTransducer * t)
+{
+    hfst::StringSet retval;
+    hfst::StringSet const & alphabet = t->get_alphabet();
+    for (hfst::StringSet::const_iterator it = alphabet.begin();
+         it != alphabet.end(); ++it) {
+        if (hfst_ol::PmatchAlphabet::is_printable(*it)) {
+            retval.insert(*it);
+        }
+    }
+    return retval;
+}
+
 HfstTransducer * make_list(HfstTransducer * t, ImplementationType f)
 {
     std::string arc = "@L.";
-    hfst::StringSet alphabet = t->get_alphabet();
+    hfst::StringSet alphabet = get_non_special_alphabet(t);
     for (hfst::StringSet::const_iterator it = alphabet.begin();
          it != alphabet.end(); ++it) {
-        if (!hfst_ol::PmatchAlphabet::is_special(*it) &&
-            *it != hfst::internal_epsilon && *it != hfst::internal_unknown &&
-            *it != hfst::internal_identity && *it != hfst::internal_default) {
-            arc.append(*it);
-            arc.append("_");
-        }
+        arc.append(*it);
+        arc.append("_");
     }
     arc.append("@");
     return new HfstTransducer(arc, f);
@@ -567,15 +669,11 @@ HfstTransducer * make_list(HfstTransducer * t, ImplementationType f)
 HfstTransducer * make_exc_list(HfstTransducer * t, ImplementationType f)
 {
     std::string arc = "@X.";
-    hfst::StringSet alphabet = t->get_alphabet();
+    hfst::StringSet alphabet = get_non_special_alphabet(t);
     for (hfst::StringSet::const_iterator it = alphabet.begin();
          it != alphabet.end(); ++it) {
-        if (!hfst_ol::PmatchAlphabet::is_special(*it) &&
-            *it != hfst::internal_epsilon && *it != hfst::internal_unknown &&
-            *it != hfst::internal_identity && *it != hfst::internal_default) {
-            arc.append(*it);
-            arc.append("_");
-        }
+        arc.append(*it);
+        arc.append("_");
     }
     arc.append("@");
     return new HfstTransducer(arc, f);
@@ -585,14 +683,10 @@ HfstTransducer * make_sigma(HfstTransducer * t)
 {
     HfstTransducer * retval =
         new HfstTransducer(format);
-    hfst::StringSet alphabet = t->get_alphabet();
+    hfst::StringSet alphabet = get_non_special_alphabet(t);
     for (hfst::StringSet::const_iterator it = alphabet.begin();
          it != alphabet.end(); ++it) {
-        if (!hfst_ol::PmatchAlphabet::is_special(*it) &&
-            *it != hfst::internal_epsilon && *it != hfst::internal_unknown &&
-            *it != hfst::internal_identity && *it != hfst::internal_default) {
             retval->disjunct(HfstTransducer(*it, format));
-        }
     }
     return retval;
 }
@@ -619,8 +713,6 @@ PmatchTransducerContainer * make_nrc_exit(void)
 { return epsilon_to_symbol_container(NRC_EXIT_SYMBOL); }
 PmatchTransducerContainer * make_nlc_exit(void)
 { return epsilon_to_symbol_container(NLC_EXIT_SYMBOL); }
-PmatchTransducerContainer * make_passthrough(void)
-{ return epsilon_to_symbol_container(PASSTHROUGH_SYMBOL); }
 
 char * get_delimited(const char *s, char delim_left, char delim_right)
 {
@@ -968,6 +1060,7 @@ void init_globals(void)
     variables["max-context-length"] = "254";
     variables["max-recursion"] =  "5000";
     variables["need-separators"] = "on";
+    variables["xerox-composition"] = "on";
     variables["vector-similarity-projection-factor"] = "1.0";
     call_stack.clear();
     def_insed_expressions.clear();
@@ -975,23 +1068,34 @@ void init_globals(void)
     unsatisfied_insertions.clear();
     used_definitions.clear();
     function_names.clear();
+    capture_names.clear();
     zero_minimization_guard();
     need_delimiters = false;
     pmatchnerrs = 0;
 }
 
+string expand_includes(const string & script)
+{
+    return string(script);
+    
+//     string filepath = hfst::pmatch::path_from_filename($1);
+}
+
 std::map<std::string, HfstTransducer*>
 compile(const string& pmatch, map<string,HfstTransducer*>& defs,
         ImplementationType impl, bool be_verbose, bool do_flatten,
+        bool do_include_cosine_distances,
         std::string includedir_)
 {
     // lock here?
     init_globals();
-    data = strdup(pmatch.c_str());
+    string expanded_script = expand_includes(pmatch);
+    data = strdup(expanded_script.c_str());
     startptr = data;
     len = strlen(data);
     verbose = be_verbose;
     flatten = do_flatten;
+    include_cosine_distances = do_include_cosine_distances;
     includedir = includedir_;
     vector_similarity_projection_factor = 1.0;
     for (map<string, HfstTransducer*>::iterator it = defs.begin();
@@ -1049,6 +1153,8 @@ compile(const string& pmatch, map<string,HfstTransducer*>& defs,
                 HfstTransducer * tmp = defs_it->second->evaluate();
                 tmp->minimize();
                 dummy.harmonize(*tmp);
+                // This is what it will be called in the archive
+                tmp->set_name(defs_it->first);
                 retval[defs_it->first] = tmp;
             }
         }
@@ -1069,10 +1175,12 @@ compile(const string& pmatch, map<string,HfstTransducer*>& defs,
             std::cerr << definitions.begin()->first << " as root\n";
             hfst::HfstTransducer * tmp = definitions.begin()->second->evaluate();
             tmp->minimize();
+            tmp->set_name("TOP");
             retval.insert(std::pair<std::string, hfst::HfstTransducer*>("TOP", tmp));
         } else {
             hfst::HfstTransducer * tmp = definitions["TOP"]->evaluate();
             tmp->minimize();
+            tmp->set_name("TOP");
             retval.insert(std::pair<std::string, hfst::HfstTransducer*>("TOP", tmp));
         }
     }
@@ -1091,8 +1199,11 @@ compile(const string& pmatch, map<string,HfstTransducer*>& defs,
         begins_and_ends_with_non_whitespace.concatenate(anything);
         begins_and_ends_with_non_whitespace.concatenate(not_whitespace);
         begins_and_ends_with_non_whitespace.compose(*(retval["TOP"]));
+        HfstTransducer is_single_non_whitespace(not_whitespace);
+        is_single_non_whitespace.compose(*(retval["TOP"]));
         HfstTransducer empty(format);
-        if (begins_and_ends_with_non_whitespace.compare(empty) == false) {
+        if (begins_and_ends_with_non_whitespace.compare(empty) == false ||
+            is_single_non_whitespace.compare(empty) == false) {
             HfstTransducer whitespace_punct_context(*(get_utils()->latin1_whitespace_acceptor));
             whitespace_punct_context.disjunct(*(get_utils()->latin1_punct_acceptor));
             whitespace_punct_context.disjunct(HfstTransducer("@BOUNDARY@", format));
@@ -1106,6 +1217,13 @@ compile(const string& pmatch, map<string,HfstTransducer*>& defs,
             top_with_boundaries->concatenate(RC);
             delete retval["TOP"];
             retval["TOP"] = add_pmatch_delimiters(top_with_boundaries);
+            (retval["TOP"])->minimize();
+            if (hfst::pmatch::verbose) {
+                double duration = (clock() - hfst::pmatch::timer) /
+                    (double) CLOCKS_PER_SEC;
+                hfst::pmatch::timer = clock();
+                std::cerr << "added automatic context separators in " << duration << " seconds\n";
+            }
         }
     }
     for(std::map<std::string, std::string>::iterator it = variables.begin();
@@ -1140,7 +1258,7 @@ HfstTransducer * read_text(std::string filename, ImplementationType type,
 {
     std::ifstream infile;
     std::string line;
-    infile.open(filename);
+    infile.open(filename.c_str());
     HfstTokenizer tok;
     HfstTransducer * retval = new HfstTransducer(type);
     if(!infile.good()) {
@@ -1192,7 +1310,7 @@ void read_vec(std::string filename)
     std::string line;
     size_t linenumber = 0;
     char separator = '\t';
-    infile.open(filename);
+    infile.open(filename.c_str());
     if(!infile.good()) {
         std::cerr << "pmatch: could not open vector file " << filename <<
             " for reading\n";
@@ -1222,8 +1340,16 @@ void read_vec(std::string filename)
             }
             // there can be one more from pos to the newline if there isn't a
             // separator at the end
+#if defined(NO_CPLUSPLUS_11)
+            if (*(line.rbegin()) != separator) {
+#else
             if (line.back() != separator) {
-                components.push_back(strtof(line.substr(pos + 1).c_str(), NULL));
+#endif
+#if defined _MSC_VER && 1200 <= _MSC_VER
+	      components.push_back((float)strtod(line.substr(pos + 1).c_str(), NULL));
+#else
+	      components.push_back(strtof(line.substr(pos + 1).c_str(), NULL));
+#endif
             }
             if (word_vectors.size() != 0 && word_vectors[0].vector.size() != components.size()) {
                 std::cerr << "pmatch warning: vector file " << filename <<
@@ -1471,6 +1597,7 @@ HfstTransducer * PmatchUtilityTransducers::cap(HfstTransducer & t, Side side, bo
         continuation2.repeat_star();
         cap.concatenate(continuation2);
         retval->compose(cap);
+        retval->output_project();
     }
     retval->minimize();
     return retval;
@@ -1786,7 +1913,11 @@ HfstTransducer * PmatchUnaryOperation::evaluate(PmatchEvalType eval_type)
     } else if (op == TermComplement) {
         HfstTransducer* any = new HfstTransducer(hfst::internal_identity,
                                                  hfst::pmatch::format);
-        any->subtract(*retval);
+        hfst::StringSet alphabet = get_non_special_alphabet(retval);
+        for (hfst::StringSet::iterator it = alphabet.begin(); it != alphabet.end(); ++it) {
+            HfstTransducer symbol(*it, hfst::pmatch::format);
+            any->subtract(symbol);
+        }
         delete retval;
         retval = any;
     } else if (op == Cap) {
@@ -1913,7 +2044,7 @@ HfstTransducer * PmatchUnaryOperation::evaluate(PmatchEvalType eval_type)
             retval->reverse();
             PmatchTransducerContainer * tmp = make_minimization_guard();
             HfstTransducer * head = tmp->evaluate(); delete tmp;
-            HfstTransducer passthrough(hfst::internal_epsilon, PASSTHROUGH_SYMBOL, format);
+            HfstTransducer passthrough(PASSTHROUGH_SYMBOL, format);
             HfstTransducer nlc_entry(hfst::internal_epsilon, NLC_ENTRY_SYMBOL, format);
             HfstTransducer nlc_exit(hfst::internal_epsilon, NLC_EXIT_SYMBOL, format);
             nlc_entry.concatenate(*retval);
@@ -1936,13 +2067,13 @@ HfstTransducer * PmatchUnaryOperation::evaluate(PmatchEvalType eval_type)
         if (!parent_is_context) {
             PmatchTransducerContainer * tmp = make_minimization_guard();
             HfstTransducer * head = tmp->evaluate(); delete tmp;
-            HfstTransducer passthrough(hfst::internal_epsilon, PASSTHROUGH_SYMBOL, format);
-            HfstTransducer nlc_entry(hfst::internal_epsilon, NLC_ENTRY_SYMBOL, format);
-            HfstTransducer nlc_exit(hfst::internal_epsilon, NLC_EXIT_SYMBOL, format);
-            nlc_entry.concatenate(*retval);
-            nlc_entry.concatenate(nlc_exit);
-            nlc_entry.disjunct(passthrough);
-            head->concatenate(nlc_entry);
+            HfstTransducer passthrough(PASSTHROUGH_SYMBOL, format);
+            HfstTransducer nrc_entry(hfst::internal_epsilon, NRC_ENTRY_SYMBOL, format);
+            HfstTransducer nrc_exit(hfst::internal_epsilon, NRC_EXIT_SYMBOL, format);
+            nrc_entry.concatenate(*retval);
+            nrc_entry.concatenate(nrc_exit);
+            nrc_entry.disjunct(passthrough);
+            head->concatenate(nrc_entry);
             delete retval;
             retval = head;
             }
