@@ -22,6 +22,7 @@
 #include <time.h>
 #include <iomanip>
 #include <cmath>
+#include <algorithm>
 #include "HfstTransducer.h"
 #include "HfstXeroxRules.h"
 #include "xre_utils.h"
@@ -49,10 +50,12 @@ extern std::set<std::string> inserted_names;
 extern std::set<std::string> unsatisfied_insertions;
 extern std::set<std::string> used_definitions;
 extern std::set<std::string> function_names;
+extern std::set<std::string> capture_names;
 extern std::vector<WordVector> word_vectors;
 extern ImplementationType format;
 extern bool verbose;
 extern bool flatten;
+extern bool include_cosine_distances;
 extern std::string includedir;
 extern clock_t timer;
 extern int minimization_guard_count;
@@ -126,6 +129,24 @@ HfstTransducer * add_pmatch_delimiters(HfstTransducer * regex);
  */
 PmatchTransducerContainer * epsilon_to_symbol_container(std::string s);
 PmatchTransducerContainer * make_end_tag(std::string tag);
+PmatchTransducerContainer * make_capture_tag(std::string tag);
+PmatchTransducerContainer * make_captured_tag(std::string tag);
+PmatchObject * make_with_tag_entry(std::string key, std::string value);
+PmatchObject * make_with_tag_exit(std::string key);
+
+std::vector<std::pair<WordVector, WordVecFloat> > get_top_n(
+    size_t n,
+    const std::vector<WordVector> & vecs,
+    WordVector & comparison_point);
+
+std::vector<std::pair<WordVector, WordVecFloat> > get_top_n_transformed(
+    size_t n,
+    const std::vector<WordVector> & vecs,
+    std::vector<WordVecFloat> plane_vec,
+    std::vector<WordVecFloat> comparison_point,
+    WordVecFloat translation_term,
+    bool negative);
+
 template<typename T> std::vector<T> pointwise_minus(std::vector<T> l,
                                                     std::vector<T> r);
 template<typename T> std::vector<T> pointwise_plus(std::vector<T> l,
@@ -137,9 +158,14 @@ template<typename T> T dot_product(std::vector<T> l,
 template<typename T> T square_sum(std::vector<T> v);
 template<typename T> T norm(std::vector<T> v);
 WordVecFloat cosine_distance(WordVector left, WordVector right);
-PmatchObject * compile_like_arc(std::string word1, std::string word2 = "",
-    unsigned int nwords = 10);
+PmatchObject * compile_like_arc(std::string word1, std::string word2,
+                                unsigned int nwords = 10, bool is_negative = false);
+PmatchObject * compile_like_arc(std::string word,
+                                unsigned int nwords = 10);
+
 PmatchTransducerContainer * make_counter(std::string name);
+
+hfst::StringSet get_non_special_alphabet(HfstTransducer * t);
 HfstTransducer * make_list(HfstTransducer * t,
                            ImplementationType f = format);
 HfstTransducer * make_exc_list(HfstTransducer * t,
@@ -177,6 +203,8 @@ double get_weight(const char* s);
 
 void init_globals(void);
 
+string expand_includes(const string & script);
+
 /**
  * @brief compile new transducer
  */
@@ -185,6 +213,7 @@ std::map<std::string, HfstTransducer*>
             std::map<std::string,hfst::HfstTransducer*>& defs,
             hfst::ImplementationType type,
             bool be_verbose = false, bool do_flatten = false,
+            bool include_cosine_distances = false,
             std::string includedir = "");
 
 void print_size_info(HfstTransducer * net);
@@ -611,6 +640,9 @@ struct PmatchTransducerContainer: public PmatchObject{
         }
         HfstTransducer * retval = new HfstTransducer(*t);
         retval->set_final_weights(hfst::double_to_float(weight), true);
+        if (name != "") {
+            retval->set_name(name);
+        }
         return retval;
     }
 };
