@@ -47,7 +47,7 @@ CLASSES:
 
 """
 
-__version__ = "3.12.1.0"
+__version__ = "3.13.0.0"
 
 import hfst.exceptions
 import hfst.sfst_rules
@@ -327,7 +327,7 @@ def regex(re, **kvargs):
             vtype = str(type(v))
             if "HfstTransducer" in vtype:
                 comp.define_transducer(k,v)
-                print('defining transducer')
+                # print('defining transducer')
             else:
                 pass
 
@@ -745,7 +745,7 @@ def compile_xfst_file(filename, **kvargs):
        if error == sys.stdout:
           arg2 == "cout"
 
-       retval = hfst_compile_xfst(xfstcomp, data, arg1, arg2)
+       retval = libhfst.hfst_compile_xfst(xfstcomp, data, arg1, arg2)
 
        if isinstance(output, StringIO):
           output.write(unicode(libhfst.get_hfst_xfst_string_one(), 'utf-8'))
@@ -756,6 +756,57 @@ def compile_xfst_file(filename, **kvargs):
       print('Parsed file with return value %i (0 indicating succesful parsing).' % retval)
     return retval
 
+def compile_twolc_file(inputfilename, outputfilename, **kvargs):
+    """
+    Compile twolc file *inputfilename* and store the result to file *outputfilename*.
+
+    Parameters
+    ----------
+    * `inputfilename` :
+        The name of the twolc input file.
+    * `outputfilename` :
+        The name of the transducer output file.
+    * `kvargs` :
+        Arguments recognized are: silent, verbose, resolve_right_conflicts, resolve_left_conflicts, type.
+    * `silent` :
+        Whether compilation is performed in silent mode, defaults to False.
+    * `verbose` :
+        Whether compilation is performed in verbose mode, defaults to False.
+    * `resolve_right_conflicts` :
+        Whether right arrow conflicts are resolved, defaults to True.
+    * `resolve_left_conflicts` :
+        Whether left arrow conflicts are resolved, defaults to False.
+    * `type` :
+        Implementation type of the compiler, defaults to hfst.get_default_fst_type().
+
+    Returns
+    -------
+    On success zero, else an integer other than zero.
+    """
+    silent=False
+    verbose=False
+    resolve_right_conflicts=True
+    resolve_left_conflicts=False
+    implementation_type=get_default_fst_type()
+
+    for k,v in kvargs.items():
+        if k == 'type':
+            implementation_type = v
+        elif k == 'silent':
+            silent=v
+        elif k == 'verbose':
+            verbose=v
+        elif k == 'resolve_right_conflicts':
+            resolve_right_conflicts=v
+        elif k == 'resolve_left_conflicts':
+            resolve_left_conflicts=v
+        else:
+            print('Warning: ignoring unknown argument %s.' % (k))
+
+    return libhfst.TwolcCompiler.compile(inputfilename, outputfilename, silent, verbose,
+                                         resolve_right_conflicts, resolve_left_conflicts,
+                                         implementation_type)
+
 def compile_pmatch_file(filename):
     """
     Compile pmatch expressions as defined in *filename* and return a tuple of
@@ -934,15 +985,27 @@ def fsa(arg):
     deftok = HfstTokenizer()
     retval = HfstBasicTransducer()
     if isinstance(arg, str):
-       retval.disjunct(deftok.tokenize(_check_word(arg)), 0)
+       if len(arg) == 0:
+           retval.set_final_weight(0, 0) # epsilon transducer with zero weight
+       else:
+           retval.disjunct(deftok.tokenize(_check_word(arg)), 0)
     elif _is_weighted_word(arg):
-       retval.disjunct(deftok.tokenize(_check_word(arg[0])), arg[1])
+       if len(arg) == 0:
+           retval.set_final_weight(0, arg[1]) # epsilon transducer with weight
+       else:
+           retval.disjunct(deftok.tokenize(_check_word(arg[0])), arg[1])
     elif isinstance(arg, tuple) or isinstance(arg, list):
        for word in arg:
            if _is_weighted_word(word):
-              retval.disjunct(deftok.tokenize(_check_word(word[0])), word[1])
+              if len(word) == 0:
+                  retval.set_final_weight(0, word[1]) # epsilon transducer with weight
+              else:
+                  retval.disjunct(deftok.tokenize(_check_word(word[0])), word[1])
            elif isinstance(word, str):
-              retval.disjunct(deftok.tokenize(_check_word(word)), 0)
+              if len(word) == 0:
+                  retval.set_final_weight(0, 0) # epsilon transducer with zero weight
+              else:
+                  retval.disjunct(deftok.tokenize(_check_word(word)), 0)
            else:
               raise RuntimeError('Tuple/list element not a string or tuple of string and weight.')
     else:
@@ -1230,6 +1293,34 @@ def intersect(transducers):
     retval.minimize()
     return retval
 
+def compose(transducers):
+    """
+    Return a composition of *transducers*.
+    """
+    retval = None
+    for tr in transducers:
+        if retval == None:
+            retval = HfstTransducer(tr)
+        else:
+            retval.compose(tr)
+    retval.minimize()
+    return retval
+
+def cross_product(transducers):
+    """
+    Return a cross product of *transducers*.
+    """
+    retval = None
+    for tr in transducers:
+        if retval == None:
+            retval = HfstTransducer(tr)
+        else:
+            retval.cross_product(tr)
+    retval.minimize()
+    return retval
+
+
+
 class ImplementationType:
     """
     Back-end implementation.
