@@ -55,6 +55,16 @@ ImplementationType output_type = hfst::UNSPECIFIED_TYPE;
 bool hfst_format = true;
 std::string options = "";
 
+void set_output_type(ImplementationType type)
+{
+  if (output_type != hfst::UNSPECIFIED_TYPE)
+    {
+      error(EXIT_FAILURE, 0,
+            "Output type defined several times.");
+    }
+  output_type = type;
+}
+
 void
 print_usage()
 {
@@ -78,7 +88,7 @@ print_usage()
     "  -Q  --quick                       When converting to optimized-lookup, don't try hard to compress\n");
     fprintf(message_out, "\n");
     print_common_unary_program_parameter_instructions(message_out);
-        fprintf(message_out, 
+        fprintf(message_out,
             "FMT must be name of a format usable by libhfst, i.e. one of the following:\n"
         "{ foma, openfst-tropical, openfst-log, sfst, xfsm\n"
         "  optimized-lookup-weighted, optimized-lookup-unweighted }.\n"
@@ -100,7 +110,7 @@ parse_options(int argc, char** argv)
         {
           HFST_GETOPT_COMMON_LONG,
           HFST_GETOPT_UNARY_LONG,
-          // add tool-specific options here 
+          // add tool-specific options here
           {"use-backend-format", no_argument, 0, 'b'},
           {"format",       required_argument, 0, 'f'},
           {"sfst",               no_argument, 0, 'S'},
@@ -114,8 +124,8 @@ parse_options(int argc, char** argv)
           {0,0,0,0}
         };
         int option_index = 0;
-        // add tool-specific options here 
-        char c = getopt_long(argc, argv, HFST_GETOPT_COMMON_SHORT
+        // add tool-specific options here
+        int c = getopt_long(argc, argv, HFST_GETOPT_COMMON_SHORT
                              HFST_GETOPT_UNARY_SHORT "SFtlOwQf:bx",
                              long_options, &option_index);
         if (-1 == c)
@@ -129,32 +139,42 @@ parse_options(int argc, char** argv)
 #include "inc/getopt-cases-common.h"
 #include "inc/getopt-cases-unary.h"
           // add tool-specific cases here
-                case 'f':
-          output_type = hfst_parse_format_name(optarg);
+        case 'f':
+          set_output_type(hfst_parse_format_name(optarg));
+#ifndef HAVE_XFSM
+          if (output_type == hfst::XFSM_TYPE)
+            error(EXIT_FAILURE, 0,
+                  "xfsm back-end is not available");
+#endif
           break;
         case 'b':
           hfst_format=false;
           break;
         case 'S':
-          output_type = hfst::SFST_TYPE;
+          set_output_type(hfst::SFST_TYPE);
           break;
         case 'F':
-          output_type = hfst::FOMA_TYPE;
+          set_output_type(hfst::FOMA_TYPE);
           break;
         case 'x':
-          output_type = hfst::XFSM_TYPE;
+#ifdef HAVE_XFSM
+          set_output_type(hfst::XFSM_TYPE);
+#else
+          error(EXIT_FAILURE, 0,
+                "xfsm back-end is not available");
+#endif
           break;
         case 't':
-          output_type = hfst::TROPICAL_OPENFST_TYPE;
+          set_output_type(hfst::TROPICAL_OPENFST_TYPE);
           break;
         case 'l':
-          output_type = hfst::LOG_OPENFST_TYPE;
+          set_output_type(hfst::LOG_OPENFST_TYPE);
           break;
         case 'O':
-          output_type = hfst::HFST_OL_TYPE;
+          set_output_type(hfst::HFST_OL_TYPE);
           break;
         case 'w':
-          output_type = hfst::HFST_OLW_TYPE;
+          set_output_type(hfst::HFST_OLW_TYPE);
           break;
     case 'Q':
         options = "quick";
@@ -165,7 +185,7 @@ parse_options(int argc, char** argv)
     
     if (output_type == hfst::UNSPECIFIED_TYPE)
     {
-        error(EXIT_FAILURE, 0, 
+        error(EXIT_FAILURE, 0,
               "You must specify an output type "
               "(one of -S, -F, -t, -x, -l, -O, or -w)");
     }
@@ -196,7 +216,7 @@ process_stream(HfstInputStream& instream, HfstOutputStream& outstream)
         char* inputname = hfst_get_name(orig, inputfilename);
         if (transducer_n == 1)
         {
-          verbose_printf("Converting %s...\n", inputname); 
+          verbose_printf("Converting %s...\n", inputname);
         }
         else
         {
@@ -238,10 +258,10 @@ int main( int argc, char **argv ) {
     {
         fclose(outfile);
     }
-    verbose_printf("Reading from %s, writing to %s\n", 
+    verbose_printf("Reading from %s, writing to %s\n",
         inputfilename, outfilename);
     char* format_description = hfst_strformat(output_type);
-    if (hfst_format && (output_type != hfst::XFSM_TYPE)) 
+    if (hfst_format && (output_type != hfst::XFSM_TYPE))
       {
         verbose_printf("Writing %s format transducers with HFST3 headers\n",
                        format_description);
@@ -267,13 +287,19 @@ int main( int argc, char **argv ) {
     try {
       instream = (inputfile != stdin) ?
         new HfstInputStream(inputfilename) : new HfstInputStream();
-    } 
+    }
     catch(const FileIsInGZFormatException e)
       {
         error(EXIT_FAILURE, 0, "%s seems to be a gzipped native foma file, you must first unzip it",
               inputfilename);
         return EXIT_FAILURE;
       }
+    catch(const ImplementationTypeNotAvailableException e)
+      {
+        error(EXIT_FAILURE, 0, "%s is in %s format which is not available",
+              inputfilename, hfst::implementation_type_to_format(e.get_type()));
+        return EXIT_FAILURE;
+      }
     catch(const HfstException e)  {
         error(EXIT_FAILURE, 0, "%s is not a valid transducer file",
               inputfilename);
